﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Kivii.Sample.ImageUploader.Controls
{
    internal class PanelEx : Panel
    {
        #region 属性
        private Rectangle _selectionBounds = Rectangle.Empty;
        /// <summary>
        /// 选择区域的XY
        /// </summary>
        public Rectangle SelectionBounds
        {
            get
            {
                return _selectionBounds;
            }
            set
            {
                _selectionBounds = value;
            }
        }
        /// <summary>
        /// 启用选择区域
        /// </summary>
        public bool EnableSelection { get; set; }

        private Color _selectionBorderColor = Color.Red;
        /// <summary>
        /// 选择区域边界颜色
        /// </summary>
        [DefaultValue(typeof(Color), "Red")]
        public Color SelectionBorderColor
        {
            get { return _selectionBorderColor; }
            set { _selectionBorderColor = value; this.Invalidate(); }
        }

        private int _selectionBorderWidth = 2;
        /// <summary>
        /// 选择区域边界线粗细
        /// </summary>
        [DefaultValue(2)]
        public int SelectionBorderWidth
        {
            get { return _selectionBorderWidth; }
            set { _selectionBorderWidth = value; this.Invalidate(); }
        }

        /// <summary>
        /// 起始偏移量
        /// </summary>
        private System.Drawing.Point _startPosition = new System.Drawing.Point(0, 0);
        /// <summary>
        /// 是否正在移动
        /// </summary>
        private int _movingStatus = 0;

        #endregion
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            if (EnableSelection)
            {
                using (Pen p = new Pen(this.SelectionBorderColor, this.SelectionBorderWidth))
                {
                    e.Graphics.DrawRectangle(p, SelectionBounds);
                }
            }
        }
        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            if (e.Button == System.Windows.Forms.MouseButtons.Left)
            {
                if (SelectionBounds.Contains(e.X, e.Y))
                {
                    _startPosition = new System.Drawing.Point(e.X, e.Y);
                    _movingStatus = 1;
                }
            }
        }
        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (_movingStatus == 0)
            {
                base.OnMouseMove(e);
                return;
            }
            Rectangle rect;
            if (_movingStatus == 2)
            {
                rect = new Rectangle(this.PointToScreen(_selectionBounds.Location), _selectionBounds.Size);
                ControlPaint.DrawReversibleFrame(rect, Color.SkyBlue, FrameStyle.Thick);
            }
            _movingStatus = 2;

            _selectionBounds.Offset(e.X - _startPosition.X, e.Y - _startPosition.Y);
            _startPosition.X = e.X;
            _startPosition.Y = e.Y;
            if (_selectionBounds.X < 0)
            {
                _selectionBounds.X = 0;
            }
            if (_selectionBounds.Y < 0)
            {
                _selectionBounds.Y = 0;
            }
            if (_selectionBounds.Right > this.ClientSize.Width)
            {
                _selectionBounds.X = this.ClientSize.Width - _selectionBounds.Width;
            }
            if (_selectionBounds.Bottom > this.ClientSize.Height)
            {
                _selectionBounds.Y = this.ClientSize.Height - _selectionBounds.Height;
            }
            rect = new Rectangle(this.PointToScreen(_selectionBounds.Location), _selectionBounds.Size);
            ControlPaint.DrawReversibleFrame(rect, Color.SkyBlue, FrameStyle.Thick);
            this.Cursor = Cursors.SizeAll;
            base.OnMouseMove(e);
        }
        protected override void OnMouseUp(MouseEventArgs e)
        {
            if (_movingStatus == 2) this.Invalidate();
            _movingStatus = 0;


            base.OnMouseUp(e);
        }

        public Image GetSelectionImage(float zoom)
        {
            if (this.BackgroundImage == null) return null;
            if (!EnableSelection) return this.BackgroundImage;

            Bitmap bmp = new Bitmap(_selectionBounds.Width, _selectionBounds.Height);
            using (Graphics g = Graphics.FromImage(bmp))
            {
                g.DrawImage(
                    this.BackgroundImage,
                    new Rectangle(0, 0, bmp.Width, bmp.Height), _selectionBounds.X / zoom, _selectionBounds.Y / zoom, _selectionBounds.Width / zoom, _selectionBounds.Height / zoom, GraphicsUnit.Pixel);
            }
            return bmp;

        }
    }
}
